#include <gtkhtml/gtkhtml.h>
#include <gtkhtml/gtkhtml-embedded.h>

#include <Rdefines.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include <unistd.h> /* for read */

/*
  Read the specified file given by the URL and copy the contents into the
  Gtk HTML stream, making it available to the widget.
  If closeStream is TRUE, we explicitly close the stream before we exit.
  When called from S, it is better to use on.exit(.Call("R_closeHTMLStream", )
 */
static double
readFileToHTMLStream(const char * url, GtkHTMLStream *stream, Rboolean closeStream)
{
	int fd;
        int ok, offset = 0;
	
	if (url) {
		url += offset;
		fd = open (url, O_RDONLY);

		if (fd != -1) {
			gchar *buf;
			size_t size;
			double total = 0;

			buf = alloca (8192);
			while ((size = read (fd, buf, 8192)) > 0) {
				total += size;
				gtk_html_stream_write (stream, buf, size);
			}
			if(closeStream) 
				gtk_html_stream_close (stream,
						       size == -1
						       ? GTK_HTML_STREAM_ERROR
						       : GTK_HTML_STREAM_OK);
			close (fd);
			return(total);
		}
	}
	if(closeStream)
		gtk_html_stream_close (stream, GTK_HTML_STREAM_ERROR);
        PROBLEM "[readFileToHTMLStream] can't open URL %s\n", url
	ERROR;

	return(-1);
}

SEXP
R_readFileToHTMLStream(SEXP url, SEXP stream, SEXP close)
{
   SEXP ans = R_NilValue;

   ans = allocVector(REALSXP, 1);
   REAL(ans)[0] = readFileToHTMLStream(CHAR(STRING_ELT(url, 0)), (GtkHTMLStream*) R_ExternalPtrAddr(stream), LOGICAL(close)[0]);
   return(ans);
}

/* 
 S-level access to close a GtkHTMLStream object. This can be called from 
 a url_requested callback for a GtkHTML widget.

 If status is TRUE, we close the stream indicating that the load was successful.
 Otherwise, we close the stream and indicate an error occurred.
*/
SEXP
R_closeHTMLStream(SEXP stream, SEXP status)
{
   gtk_html_stream_close((GtkHTMLStream*) R_ExternalPtrAddr(stream), 
			   LOGICAL(status)[0] ? GTK_HTML_STREAM_OK : GTK_HTML_STREAM_ERROR);

   return(R_NilValue);
}


/*
  These two routines are not needed as they are the low-level way to explicitly
  by-pass S as the callback language. It is more flexible and easier to understand
  in many ways to specify an R function as the callback for the url_requested
  signal and then to have that function call the routines above to implement the
  details of the callback.
 */
static void
url_requested (GtkHTML *html, const char *url, GtkHTMLStream *stream, gpointer userData)
{
   readFileToHTMLStream(url, stream, TRUE);
}

SEXP
R_setURLHandler(SEXP swidget)
{
  GtkWidget *w = (GtkWidget *) R_ExternalPtrAddr(swidget);
  gtk_signal_connect(GTK_OBJECT(w), "url_requested", url_requested, NULL);

  return(R_NilValue);
}

