#include "gtkUtils.h"
#include "RGtkHTML.h"

#if 0
#include "gtkhtml/htmlaccess.h"
#else
#include "gtkhtml/htmlengine.h"
#include "gtkhtml/htmlform.h"
#endif

#include "gtkhtml/htmlembedded.h"
#include "gtkhtml/htmlhidden.h"
#include "gtkhtml/htmltype.h"

USER_OBJECT_
R_getHTMLForms(USER_OBJECT_ s_html)
{
   USER_OBJECT_ ans, className, names, el;
   GtkHTML * html = GTK_HTML ((GtkObject*) getPtrValue( s_html )) ;  
   GList *ptr;
   HTMLForm *form;
   int n;

   HTMLEngine *eng;
   eng = html->engine;
   ptr = eng->formList;

   n = g_list_length(ptr);
   PROTECT(ans = NEW_LIST(n));
   PROTECT(names = NEW_CHARACTER(n));
   n = 0;
   PROTECT(className = NEW_CHARACTER(1));
   SET_STRING_ELT(className, 0, COPY_TO_USER_STRING("GtkHTMLForm"));

   while(ptr) {
           form = (HTMLForm *)ptr->data;
	   PROTECT(el = R_MakeExternalPtr(form, Rf_install("GtkHTMLForm"), R_NilValue));
	   SET_CLASS(el, className);
	   SET_VECTOR_ELT(ans, n, el);	   
	   UNPROTECT(1);
	   
           if(form->elements) {
	      HTMLEmbedded *e =(HTMLEmbedded*) form->elements->data;
	      SET_STRING_ELT(names, n, COPY_TO_USER_STRING(e->name));	      
	   }
	   n++;
	   ptr = ptr->next;
   }
   
   SET_NAMES(ans, names);
   UNPROTECT(3);   

   return(ans);
}

enum {R_ACTION, R_METHOD, R_ELEMENTS, R_VALUES, R_HIDDEN, R_TYPES};

USER_OBJECT_
R_getForm(USER_OBJECT_ s_form)
{
   HTMLForm *form = (HTMLForm*) ((GtkObject*) getPtrValue( s_form )) ;  
   int i, n;
   GList *ptr;
   gchar *tmp;
   USER_OBJECT_ ans, names, tmp1, vals, types, typeNames;

   char *SlotNames[] = {"action", "method", "elements", "values", "hidden", "types"};
   int numSlots = sizeof(SlotNames)/sizeof(SlotNames[0]);

   PROTECT(ans = NEW_LIST(numSlots));
   
   if(form->action)
     SET_VECTOR_ELT(ans, R_ACTION, mkString(form->action));

   if(form->method)
     SET_VECTOR_ELT(ans, R_METHOD, mkString(form->method));

   ptr = form->elements;
   n = g_list_length(ptr);
   PROTECT(tmp1 = NEW_LIST(n));
   PROTECT(names = NEW_CHARACTER(n));
   PROTECT(vals = NEW_CHARACTER(n));

   PROTECT(typeNames = NEW_CHARACTER(n));
   PROTECT(types = NEW_INTEGER(n));
   for(i = 0 ; i < n ; i++, ptr = ptr->next) {
      HTMLEmbedded *e = HTML_EMBEDDED(ptr->data);
      SET_VECTOR_ELT(tmp1, i, R_gtkWidgetReference((GtkWidget *)e, "HTMLEmbedded"));
      SET_STRING_ELT(names, i, COPY_TO_USER_STRING(e->name));

      if(e->value)
          SET_STRING_ELT(vals, i, COPY_TO_USER_STRING(e->value));

      INTEGER_DATA(types)[i] = e->object.klass->type;
      SET_STRING_ELT(typeNames, i, COPY_TO_USER_STRING(html_type_name(e->object.klass->type)));
   }

   SET_NAMES(tmp1, names);
   SET_VECTOR_ELT(ans, R_ELEMENTS, tmp1);
   SET_NAMES(vals, names);
   SET_VECTOR_ELT(ans, R_VALUES, vals);

   SET_NAMES(types, typeNames);
   SET_VECTOR_ELT(ans, R_TYPES, types);
   UNPROTECT(5);

   ptr = form->hidden;
   n = g_list_length(ptr);
   PROTECT(tmp1 = NEW_LIST(n));
   PROTECT(names = NEW_CHARACTER(n));
   for(i = 0 ; i < n ; i++, ptr = ptr->next) {
      HTMLHidden *e = HTML_HIDDEN(ptr->data);
      SET_VECTOR_ELT(tmp1, i, R_gtkWidgetReference((GtkWidget*)e, "HTMLHidden"));
      SET_STRING_ELT(names, i, COPY_TO_USER_STRING(HTML_EMBEDDED(e)->name));
   }
   SET_NAMES(tmp1, names);
   SET_VECTOR_ELT(ans, R_HIDDEN, tmp1);
   UNPROTECT(2);

   PROTECT(names = NEW_CHARACTER(numSlots));
   for(i = 0; i < numSlots; i++) {
     SET_STRING_ELT(names, i, COPY_TO_USER_STRING(SlotNames[i]));
   }

   SET_NAMES(ans, names);

   UNPROTECT(2);
   return(ans);
}

USER_OBJECT_
R_htmlEmbeddedReset(USER_OBJECT_ el)
{
   HTMLEmbedded *e = HTML_EMBEDDED(getPtrValue(el));
 
   if(!e) {
        PROBLEM "NULL value passed for HTMLEmbedded object"
	ERROR;
   }

   html_embedded_reset(e);
   return(R_NilValue);
}

USER_OBJECT_
R_htmlEmbeddedWidget(USER_OBJECT_ el)
{
   HTMLEmbedded *e = HTML_EMBEDDED(getPtrValue(el));
   GtkWidget *w;
   if(!e) {
        PROBLEM "NULL value passed for HTMLEmbedded object"
	ERROR;
   }

   w = html_embedded_get_widget(e);
   return(R_gtkWidgetReference((GtkWidget*)w, "GtkObject"));
}
